@@ -1,5 +1,3 @@
-from matplotlib.transforms import Bbox
-from aperturedb import ParallelLoader
 from aperturedb import CSVParser
 
 HEADER_X_POS = "x_pos"
@@ -46,7 +44,7 @@ class BBoxDataCSV(CSVParser.CSVParser):
     ``` python
 
         data = BBoxDataCSV("/path/to/BoundingBoxesData.csv")
-        loader = ParallelLoader(db)
+        loader = ParallelLoader(client)
         loader.ingest(data)
     ```
 
