@@ -25,7 +25,8 @@
 # THE SOFTWARE.
 #
 from __future__ import annotations
-from . import queryMessage_pb2
+from typing import Optional
+from . import queryMessage
 import sys
 import os
 import socket
@@ -42,6 +43,7 @@ from threading import Lock
 from types import SimpleNamespace
 from dataclasses import dataclass
 from aperturedb.Configuration import Configuration
+from aperturedb.types import CommandResponses
 
 logger = logging.getLogger(__name__)
 
@@ -82,12 +84,18 @@ class Connector(object):
     **Class to facilitate connections with an instance of ApertureDB**
 
     It lets the client execute any JSON query based on the [ApertureDB query language specification](/query_language/Overview/API%20Description)
+    It manages the TCP connection to the database.
+
+    :::note
+    - The connection is established only when a query is run.
+    - A new connection is established for each instance that runs a query, and gets closed only at destruction.
+    :::
 
     Args:
         str (host): Address of the host to connect to.
         int (port): Port to connect to.
         str (user): Username to specify while establishing a connection.
-        str (password): Password to specify while connecting to the db.
+        str (password): Password to specify while connecting to ApertureDB.
         str (token): Token to use while connecting to the database.
         bool (use_ssl): Use SSL to encrypt communication with the database.
         bool (use_keepalive): Set keepalive on the connection with the database.
@@ -105,7 +113,7 @@ class Connector(object):
                  use_keepalive=True,
                  retry_interval_seconds=1,
                  retry_max_attempts=3,
-                 config: Configuration = None):
+                 config: Optional[Configuration] = None):
         """
         Constructor for the Connector class.
         """
@@ -161,17 +169,13 @@ class Connector(object):
     def authenticate(self, shared_data, user, password, token):
         """
         Authenticate with the database. This will be called automatically from query.
-        This is separate from session refresh mechanism, and is called only once.
-        Failure leads to exception.
+        This is separate from session refresh mechanism, and is set to be called only once per session.
+        If a Refresh token also fails, this will be called again.
         """
         if not self.authenticated:
             if shared_data.session is None:
                 self.shared_data.lock = Lock()
-                try:
-                    self._authenticate(user, password, token)
-                except Exception as e:
-                    raise UnauthenticatedException(
-                        "Authentication failed:", str(e))
+                self._authenticate(user, password, token)
             else:
                 self.shared_data = shared_data
             self.authenticated = True
@@ -182,6 +186,11 @@ class Connector(object):
             self.connected = False
 
     def _send_msg(self, data):
+        # aperturedb's param ADB_MAX_CONNECTION_MESSAGE_SIZE_MB = 256 by default
+        if len(data) > (256 * 2**20):
+            logger.warning(
+                "Message sent is larger than default for ApertureDB Server. Server may disconnect.")
+
         sent_len = struct.pack('@I', len(data))  # send size first
         x = self.conn.send(sent_len + data)
         return x == len(data) + 4
@@ -260,6 +269,16 @@ class Connector(object):
         if isinstance(response, list):
             session_info = response[0]["RefreshToken"]
             if session_info["status"] != 0:
+                # Refresh token failed, we need to re-authenticate
+                # This is possible with a long lived connector, where
+                # the session token and the refresh token have expired.
+                self.authenticated = False
+                self.should_authenticate = True
+                self.shared_data.session = None
+                self.authenticate(self.shared_data,
+                                  self.config.username,
+                                  self.config.password,
+                                  self.token)
                 raise UnauthorizedException(response)
 
             self.shared_data.session = Session(
@@ -272,7 +291,6 @@ class Connector(object):
             raise UnauthorizedException(response)
 
     def _connect(self):
-
         self.conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.conn.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 1)
         if self.use_keepalive:
@@ -322,10 +340,6 @@ class Connector(object):
                 self.conn = self.context.wrap_socket(self.conn)
 
         except BaseException as e:
-            logger.error(
-                f"Error connecting to server: {str(e)} {self.config}",
-                exc_info=True,
-                stack_info=True)
             self.conn.close()
             self.connected = False
             raise
@@ -342,7 +356,7 @@ class Connector(object):
                 self._connect()
             except socket.error as e:
                 logger.error(
-                    f"Error connecting to server: {self.config} \r\n{details}.",
+                    f"Error connecting to server: {self.config} \r\n{details}. {e=}",
                     exc_info=True,
                     stack_info=True)
 
@@ -354,7 +368,7 @@ class Connector(object):
         else:
             query_str = query
 
-        query_msg = queryMessage_pb2.queryMessage()
+        query_msg = queryMessage.queryMessage()
         # query has .json and .blobs
         query_msg.json = query_str
 
@@ -375,8 +389,8 @@ class Connector(object):
                 if self._send_msg(data):
                     response = self._recv_msg()
                     if response is not None:
-                        querRes = queryMessage_pb2.queryMessage()
-                        querRes.ParseFromString(response)
+                        querRes = queryMessage.queryMessage()
+                        queryMessage.ParseFromString(querRes, response)
                         response_blob_array = [b for b in querRes.blobs]
                         self.last_response = json.loads(querRes.json)
                         break
@@ -431,8 +445,13 @@ class Connector(object):
             if try_resume:
                 self._renew_session()
         if tries == self.config.retry_max_attempts:
+            # We have tried enough times, and failed. Log some state info.
             raise Exception(
-                f"Could not query apertureDB using TCP.")
+                f"Could not query apertureDB using TCP. \r\n\
+                {self.connected=}\r\n \
+                {self.authenticated=} \r\n \
+                attempts={tries}/{self.config.retry_max_attempts} \r\n \
+                {self.config=}")
         return (self.last_response, response_blob_array)
 
     def query(self, q, blobs=[]):
@@ -494,16 +513,15 @@ class Connector(object):
                 time.sleep(1)
                 count += 1
 
-    def create_new_connection(self) -> Connector:
+    def clone(self) -> Connector:
         """
-        Create a new connection object with the same parameters as the current one.
-        This is important in multi-threaded applications, where each thread should have its own connection object.
-        Because the connection object is not thread-safe, it is not safe to share it between threads.
+        Create a new Connector object with the same parameters as the current one.
+        This is important in multi-threaded applications, where each thread should have its own Connector object.
         Be cautious when using this method, as it will create a new connection to the database, which will consume resources.
         Ideally, this method should be called once for each thread.
 
         Returns:
-            connection: Clone of original connection
+            Connector: Clone of original Connector
         """
         return type(self)(
             self.host,
@@ -514,6 +532,12 @@ class Connector(object):
             use_ssl=self.use_ssl,
             shared_data=self.shared_data)
 
+    def create_new_connection(self):
+        from aperturedb.CommonLibrary import issue_deprecation_warning
+        issue_deprecation_warning(
+            "Connector.create_new_connection", "Connector.clone")
+        return self.clone()
+
     def get_last_response_str(self):
 
         return json.dumps(self.last_response, indent=4, sort_keys=False)
@@ -540,11 +564,21 @@ class Connector(object):
 
         return self.check_status(self.response) >= 0
 
-    def check_status(self, json_res):
+    def check_status(self, json_res: CommandResponses) -> int:
+        """
+        Returns the status of the first command response from the server.
+        Can traverse a JSON recursively to find the first status.
 
-        status = 0
-        if (isinstance(self.response, dict)):
-            if ("status" not in self.response):
+        Args:
+            json_res (CommandResponses): The actual response from the server.
+
+        Returns:
+            int: The value recieved from the server, or -2 if not found.
+        """
+        # Default status is -2, which is an error, but not a server error.
+        status = -2
+        if (isinstance(json_res, dict)):
+            if ("status" not in json_res):
                 status = self.check_status(json_res[list(json_res.keys())[0]])
             else:
                 status = json_res["status"]
