@@ -41,7 +41,7 @@ class DaskManager:
         self._client.close()
         self._cluster.close()
 
-    def run(self, QueryClass: type[ParallelQuery], db: Connector, generator, batchsize, stats):
+    def run(self, QueryClass: type[ParallelQuery], client: Connector, generator, batchsize, stats):
         def process(df, host, port, use_ssl, session, connnector_type):
             metrics = Stats()
             # Dask reads data in partitions, and the first partition is of 2 rows, with all
@@ -55,12 +55,12 @@ class DaskManager:
                 shared_data = SimpleNamespace()
                 shared_data.session = session
                 shared_data.lock = Lock()
-                db = connnector_type(host=host, port=port,
-                                     use_ssl=use_ssl, shared_data=shared_data)
+                client = connnector_type(host=host, port=port,
+                                         use_ssl=use_ssl, shared_data=shared_data)
             except Exception as e:
                 logger.exception(e)
             #from aperturedb.ParallelLoader import ParallelLoader
-            loader = QueryClass(db)
+            loader = QueryClass(client)
             for i in range(0, len(df), batchsize):
                 end = min(i + batchsize, len(df))
                 slice = df[i:end]
@@ -80,15 +80,15 @@ class DaskManager:
             return metrics
 
         start_time = time.time()
-        # Passing DB as an argument to function is not supported by Dask,
-        # so we pass session and host/port instead.
+        # Connector cannot be serialized across processes,
+        # so we pass session and host/port information instead.
         computation = generator.df.map_partitions(
             process,
-            db.host,
-            db.port,
-            db.use_ssl,
-            db.shared_data.session,
-            type(db))
+            client.host,
+            client.port,
+            client.use_ssl,
+            client.shared_data.session,
+            type(client))
         computation = computation.persist()
         if stats:
             progress(computation)
