@@ -1,8 +1,5 @@
-import time
-import requests
 import boto3
 
-import numpy as np
 import cv2
 
 from aperturedb import CSVParser
@@ -158,7 +155,7 @@ class ImageDataCSV(CSVParser.CSVParser, ImageDataProcessor):
     ``` python
 
         data = ImageDataCSV("/path/to/ImageData.csv")
-        loader = ParallelLoader(db)
+        loader = ParallelLoader(client)
         loader.ingest(data)
     ```
 
@@ -365,7 +362,7 @@ class ImageSparseAddDataCSV(SparseAddingDataCSV, ImageDataProcessor):
     ```python
 
         data = ImageSparseAddDataCSV("/path/to/ImageData.csv")
-        loader = ParallelLoader(db)
+        loader = ParallelLoader(client)
         loader.ingest(data)
     ```
     """
