@@ -1,121 +1,19 @@
 from __future__ import annotations
-from typing import Callable, Dict, List, Tuple, Optional
+from typing import List, Tuple
 from aperturedb import Parallelizer
 import numpy as np
-import json
 import logging
-import math
+import inspect
 
 
 from aperturedb.DaskManager import DaskManager
 from aperturedb.Connector import Connector
 from aperturedb.types import Commands, Blobs, CommandResponses
+from aperturedb.CommonLibrary import execute_query
 
 logger = logging.getLogger(__name__)
 
 
-def execute_batch(q: Commands, blobs: Blobs, db: Connector,
-                  success_statuses: list[int] = [0],
-                  response_handler: Optional[Callable] = None, commands_per_query: int = 1, blobs_per_query: int = 0,
-                  strict_response_validation: bool = False) -> Tuple[int, CommandResponses, Blobs]:
-    """
-    Execute a batch of queries, doing useful logging around it.
-    Calls the response handler if provided.
-    This should be used (without the parallel machinery) instead of db.query to keep the response handling consistent, better logging, etc.
-
-    Args:
-        q (Commands): List of commands to execute.
-        blobs (Blobs): List of blobs to send.
-        db (Connector): The database connector.
-        success_statuses (list[int], optional): The list of success statuses. Defaults to [0].
-        response_handler (Callable, optional): The response handler. Defaults to None.
-        commands_per_query (int, optional): The number of commands per query. Defaults to 1.
-        blobs_per_query (int, optional): The number of blobs per query. Defaults to 0.
-        strict_response_validation (bool, optional): Whether to strictly validate the response. Defaults to False.
-
-    Returns:
-        int: The result code.
-            - 0 : if all commands succeeded
-            - 1 : if there was -1 in the response
-            - 2 : For any other code.
-        CommandResponses: The response.
-        Blobs: The blobs.
-    """
-    result = 0
-    logger.debug(f"Query={q}")
-    r, b = db.query(q, blobs)
-    logger.debug(f"Response={r}")
-
-    if db.last_query_ok():
-        if response_handler is not None:
-            # We could potentially always call this handler function
-            # and let the user deal with the error cases.
-            blobs_returned = 0
-            for i in range(math.ceil(len(q) / commands_per_query)):
-                start = i * commands_per_query
-                end = start + commands_per_query
-                blobs_start = i * blobs_per_query
-                blobs_end = blobs_start + blobs_per_query
-
-                b_count = 0
-                for req, resp in zip(q[start:end], r[start:end]):
-                    for k in req:
-                        # Ref to https://docs.aperturedata.io/query_language/Reference/shared_command_parameters/blobs
-                        blobs_where_default_true = \
-                            k in ["FindImage", "FindBlob", "FindVideo"] and (
-                                "blobs" not in req[k] or req[k]["blobs"])
-                        blobs_where_default_false = \
-                            k in [
-                                "FindDescriptor", "FindBoundingBox"] and "blobs" in req[k] and req[k]["blobs"]
-                        if blobs_where_default_true or blobs_where_default_false:
-                            count = resp[k]["returned"]
-                            b_count += count
-
-                try:
-                    # The returned blobs need to be sliced to match the
-                    # returned entities per command in query.
-                    response_handler(
-                        q[start:end],
-                        blobs[blobs_start:blobs_end],
-                        r[start:end],
-                        b[blobs_returned:blobs_returned + b_count] if len(b) >= blobs_returned + b_count else None)
-                except BaseException as e:
-                    logger.exception(e)
-                    if strict_response_validation:
-                        raise e
-                blobs_returned += b_count
-    else:
-        # Transaction failed entirely.
-        logger.error(f"Failed query = {q} with response = {r}")
-        result = 1
-
-    statuses = {}
-    if isinstance(r, dict):
-        statuses[r['status']] = [r]
-    elif isinstance(r, list):
-        # add each result to a list of the responses, keyed by the response
-        # code.
-        [statuses.setdefault(result[cmd]['status'], []).append(result)
-         for result in r for cmd in result]
-    else:
-        logger.error("Response in unexpected format")
-        result = 1
-
-    # last_query_ok means result status >= 0
-    if result != 1:
-        warn_list = []
-        for status, results in statuses.items():
-            if status not in success_statuses:
-                for wr in results:
-                    warn_list.append(wr)
-        if len(warn_list) != 0:
-            logger.warning(
-                f"Partial errors:\r\n{json.dumps(q)}\r\n{json.dumps(warn_list)}")
-            result = 2
-
-    return result, r, b
-
-
 class ParallelQuery(Parallelizer.Parallelizer):
     """
     **Parallel and Batch Querier for ApertureDB**
@@ -124,7 +22,7 @@ class ParallelQuery(Parallelizer.Parallelizer):
     so that they may be processed using different threads.
 
     Args:
-        db (Connector): The database connector.
+        client (Connector): The database connector.
         dry_run (bool, optional): Whether to run in dry run mode. Defaults to False.
     """
 
@@ -139,11 +37,17 @@ class ParallelQuery(Parallelizer.Parallelizer):
     def getSuccessStatus(cls):
         return cls.success_statuses
 
-    def __init__(self, db: Connector, dry_run: bool = False):
-
+    def __init__(self, client: Connector, dry_run: bool = False):
         super().__init__()
+        test_string = f"Connection test successful with {client.config}"
+        try:
+            _, _ = client.query([{"GetSchema": {}}], [])
+            logger.info(test_string)
+        except Exception as e:
+            logger.error(test_string.replace("successful", "failed"))
+            raise
 
-        self.db = db.create_new_connection()
+        self.client = client.clone()
 
         self.dry_run = dry_run
 
@@ -154,7 +58,7 @@ class ParallelQuery(Parallelizer.Parallelizer):
         self.commands_per_query = 1
         self.blobs_per_query = 0
         self.daskManager = None
-        self.batch_command = execute_batch
+        self.batch_command = execute_query
 
     def generate_batch(self, data: List[Tuple[Commands, Blobs]]) -> Tuple[Commands, Blobs]:
         """
@@ -184,14 +88,23 @@ class ParallelQuery(Parallelizer.Parallelizer):
                     values["image_ref"] = updates[values["image_ref"]]
                 if "video_ref" in values:
                     values["video_ref"] = updates[values["video_ref"]]
-                if "is_connected_to" in values and "_ref" in values["is_connected_to"]:
-                    values["is_connected_to"]["_ref"] = updates[values["is_connected_to"]["_ref"]]
+                if "is_connected_to" in values:
+                    if "ref" in values["is_connected_to"]:
+                        values["is_connected_to"]["ref"] = updates[values["is_connected_to"]["ref"]]
+                    for op in ["any", "all"]:
+                        if op in values["is_connected_to"]:
+                            for idx in range(len(values["is_connected_to"][op])):
+                                if "ref" in values["is_connected_to"][op][idx]:
+                                    values["is_connected_to"][op][idx]["ref"] = updates[values["is_connected_to"][op][idx]["ref"]]
                 if "connect" in values and "ref" in values["connect"]:
                     values["connect"]["ref"] = updates[values["connect"]["ref"]]
                 if "src" in values:
                     values["src"] = updates[values["src"]]
                 if "dst" in values:
                     values["dst"] = updates[values["dst"]]
+                if "ref" in values:
+                    values["ref"] = updates[values["ref"]]
+
             return batched_commands
 
         q = update_refs([cmd for query in data for cmd in query[0]])
@@ -205,12 +118,12 @@ class ParallelQuery(Parallelizer.Parallelizer):
         except BaseException as e:
             logger.exception(e)
 
-    def do_batch(self, db: Connector, data: List[Tuple[Commands, Blobs]]) -> None:
+    def do_batch(self, client: Connector, batch_start: int,  data: List[Tuple[Commands, Blobs]]) -> None:
         """
         Executes batch of queries and blobs in the database.
 
         Args:
-            db (Connector): The database connector.
+            client (Connector): The database connector.
             data (list[tuple[Commands, Blobs]]): The data to be batched.  Each tuple contains a list of commands and a list of blobs.
 
         It also provides a way for invoking a user defined function to handle the
@@ -227,7 +140,7 @@ class ParallelQuery(Parallelizer.Parallelizer):
                 def process_responses(requests, input_blobs, responses, output_blobs):
                     self.requests.extend(requests)
                     self.responses.extend(responses)
-            loader = ParallelLoader(self.db)
+            loader = ParallelLoader(self.client)
             generator = MyQueries()
             loader.ingest(generator)
         ```
@@ -244,17 +157,31 @@ class ParallelQuery(Parallelizer.Parallelizer):
                 response_handler = self.generator.response_handler
             if hasattr(self.generator, "strict_response_validation") and isinstance(self.generator.strict_response_validation, bool):
                 strict_response_validation = self.generator.strict_response_validation
+
+            # if response_handler doesn't support index, just discard the index with a wrapper.
+            if response_handler is not None:
+                parameter_count = len(inspect.signature(
+                    response_handler).parameters)
+                if parameter_count < 4 or parameter_count > 5:
+                    raise Exception("Bad Signature for response_handler :"
+                                    f"expected 6 > args > 3, got {parameter_count}")
+                if parameter_count == 4:
+                    indexless_handler = response_handler
+                    def response_handler(query, qblobs, resp, rblobs, qindex): return indexless_handler(
+                        query, qblobs, resp, rblobs)
+
             result, r, b = self.batch_command(
+                client,
                 q,
                 blobs,
-                db,
                 ParallelQuery.success_statuses,
                 response_handler,
                 self.commands_per_query,
                 self.blobs_per_query,
-                strict_response_validation=strict_response_validation)
+                strict_response_validation=strict_response_validation,
+                cmd_index=batch_start)
             if result == 0:
-                query_time = db.get_last_query_time()
+                query_time = client.get_last_query_time()
                 worker_stats["succeeded_commands"] = len(q)
                 worker_stats["succeeded_queries"] = len(data)
                 worker_stats["objects_existed"] = sum(
@@ -274,8 +201,11 @@ class ParallelQuery(Parallelizer.Parallelizer):
                     [v['status'] == 2 for i in r for k, v in filter_per_group(i)])
                 sq = 0
                 for i in range(0, len(r), self.commands_per_query):
-                    if all([v['status'] == 0 for j in r[i:i + self.commands_per_query] for k, v in filter_per_group(j)]):
-                        sq += 1
+                    # Some errors stop the whole query from being executed
+                    # https://docs.aperturedata.io/query_language/Overview/Responses#return-status
+                    if issubclass(type(r), list):
+                        if all([v['status'] == 0 for j in r[i:i + self.commands_per_query] for k, v in filter_per_group(j)]):
+                            sq += 1
                 worker_stats["succeeded_queries"] = sq
         else:
             query_time = 1
@@ -284,31 +214,35 @@ class ParallelQuery(Parallelizer.Parallelizer):
         self.times_arr.append(query_time)
         self.actual_stats.append(worker_stats)
 
-    def worker(self, thid: int, generator, start: int, end: int):
+    def worker(self, thid: int, generator, start: int, end: int, run_event) -> None:
         # A new connection will be created for each thread
-        db = self.db.create_new_connection()
+        client = self.client.clone()
 
         total_batches = (end - start) // self.batchsize
 
         if (end - start) % self.batchsize > 0:
             total_batches += 1
 
-        logger.info(f"Worker {thid} executing {total_batches} batches")
+        logger.info(
+            f"Worker {thid} executing {total_batches} batches, {self.stats=}")
         for i in range(total_batches):
-
+            if not run_event.is_set():
+                break
             batch_start = start + i * self.batchsize
             batch_end = min(batch_start + self.batchsize, end)
 
             try:
-                self.do_batch(db, generator[batch_start:batch_end])
+                self.do_batch(client, batch_start,
+                              generator[batch_start:batch_end])
             except Exception as e:
                 logger.exception(e)
                 logger.warning(
                     f"Worker {thid} failed to execute batch {i}: [{batch_start},{batch_end}]")
                 self.error_counter += 1
 
-            if thid == 0 and self.stats:
-                self.pb.update((i + 1) / total_batches)
+            if self.stats:
+                self.pb.update(self.batchsize)
+        logger.info(f"Worker {thid} executed {total_batches} batches")
 
     def get_objects_existed(self) -> int:
         return sum([stat["objects_existed"]
@@ -343,7 +277,7 @@ class ParallelQuery(Parallelizer.Parallelizer):
 
         if use_dask:
             results, self.total_actions_time = self.daskmanager.run(
-                self.__class__, self.db, generator, batchsize, stats=stats)
+                self.__class__, self.client, generator, batchsize, stats=stats)
             self.actual_stats = []
             for result in results:
                 if result is not None:
@@ -376,7 +310,7 @@ class ParallelQuery(Parallelizer.Parallelizer):
             logger.info(
                 f"Commands per query = {self.commands_per_query}, Blobs per query = {self.blobs_per_query}"
             )
-            self.run(generator, batchsize, numthreads, stats)
+            self.batched_run(generator, batchsize, numthreads, stats)
 
     def print_stats(self) -> None:
 
@@ -423,3 +357,10 @@ class ParallelQuery(Parallelizer.Parallelizer):
             print("Sample of data to be ingested:")
             for i in range(sample_count):
                 print(self.generator[i])
+
+
+def execute_batch(q: Commands, blobs: Blobs, db: Connector, *args, **kwargs) -> Tuple[int, CommandResponses, Blobs]:
+    from aperturedb.CommonLibrary import execute_query, issue_deprecation_warning
+    issue_deprecation_warning(
+        "ParallelQuery.execute_batch", "CommonLibrary.execute_query")
+    return execute_query(db, q, blobs, *args, **kwargs)
