@@ -7,6 +7,10 @@ class Order(Enum):
 
 
 class Sort():
+    """
+    **Specification of the sort order**
+    """
+
     def __init__(self, key: str, order: Order) -> None:
         self._sort = {
             "key": key,
