@@ -1,112 +1,49 @@
 """
 Miscellaneous utility functions for ApertureDB.
+This class contains a collection of helper functions to interact with the database.
 """
+from aperturedb.Query import QueryBuilder
+from aperturedb.CommonLibrary import execute_query
+from tqdm import tqdm
+from aperturedb.Connector import Connector
 import logging
 import json
-import os
-import importlib
-import sys
-from typing import List
+from typing import List, Optional, Dict
 
-from graphviz import Source, Digraph
+HAS_GRAPHVIZ = True
+try:
+    from graphviz import Source, Digraph
+except:
+    HAS_GRAPHVIZ = False
+
+    class Source:
+        pass
+
+    class Digraph:
+        pass
 
-from aperturedb.Connector import Connector
-from aperturedb.ConnectorRest import ConnectorRest
-from aperturedb import ProgressBar
-from aperturedb.ParallelQuery import execute_batch
-from aperturedb.Configuration import Configuration
-from aperturedb.cli.configure import ls
-from aperturedb.Query import QueryBuilder
 
 logger = logging.getLogger(__name__)
 
 DESCRIPTOR_CLASS = "_Descriptor"
 DESCRIPTOR_CONNECTION_CLASS = "_DescriptorSetToDescriptor"
-
 DEFAULT_METADATA_BATCH_SIZE = 100_000
 
 
-def import_module_by_path(filepath):
-    """
-    This function imports a module given a path to a python file.
-    """
-    module_name = os.path.basename(filepath)[:-3]
-    spec = importlib.util.spec_from_file_location(module_name, filepath)
-    module = importlib.util.module_from_spec(spec)
-    sys.modules[module_name] = module
-    spec.loader.exec_module(module)
-    return module
-
-
-def __create_connector(configuration: Configuration):
-    if configuration.use_rest:
-        connector = ConnectorRest(
-            host=configuration.host,
-            port=configuration.port,
-            user=configuration.username,
-            password=configuration.password,
-            config=configuration)
-    else:
-        connector = Connector(
-            host=configuration.host,
-            port=configuration.port,
-            user=configuration.username,
-            password=configuration.password,
-            config=configuration)
-    logger.info(f"Connected Using: {configuration}")
-    return connector
-
-
-def create_connector():
-    """
-    **Create a connector to the database.**
-
-    This function reads the active configuration.
-    See :ref:`adb config <adb_config>`_ command-line tool for more information.
-
-    Args:
-        None
-
-    Returns:
-        Connector: The connector to the database.
-    """
-    all_configs = ls(log_to_console=False)
-
-    ac = all_configs["active"]
-    config = None
-    if ac is not None:
-        # check if the active config is in the global or local
-        # Local should be the final choice
-        if "global" in all_configs and ac in all_configs["global"]:
-            config = all_configs["global"][ac]
-        if "local" in all_configs and ac in all_configs["local"]:
-            config = all_configs["local"][ac]
-    assert config is not None, "No active configuration found."
-
-    env_config = os.environ.get("APERTUREDB_CONFIG")
-    if env_config is not None:
-        # TODO test me.
-        config = all_configs["global"][env_config] if env_config in all_configs["global"] else all_configs["local"][env_config]
-        return __create_connector(config)
-    # Then check if the local config has active
-    else:
-        return __create_connector(config)
-
-
 class Utils(object):
     """
-    **A bunch of helper methods to get information from aperturedb.**
+    **Helper methods to get information from aperturedb, or affect it's state.**
 
     Args:
         object (Connector): The underlying Connector.
     """
 
-    def __init__(self, connector: Connector, verbose=False):
-        self.connector = connector.create_new_connection()
+    def __init__(self, client: Connector, verbose=False):
+        self.client: Connector = client.clone()
         self.verbose = verbose
 
     def __repr__(self):
-        return self.status()
+        return f"{id(self)}"
 
     def print(self, str):
         if self.verbose:
@@ -126,10 +63,10 @@ class Utils(object):
             blobs: The blobs returned by the query.
         """
         try:
-            rc, r, b = execute_batch(
-                query, blobs, self.connector, success_statuses=success_statuses)
+            rc, r, b = execute_query(self.client,
+                                     query, blobs, success_statuses=success_statuses)
         except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
+            logger.error(self.client.get_last_response_str())
             raise e
 
         if rc != 0:
@@ -147,7 +84,7 @@ class Utils(object):
 
         self.execute(q)
 
-        return self.connector.get_last_response_str()
+        return self.client.get_last_response_str()
 
     def print_schema(self, refresh=False):
 
@@ -157,7 +94,7 @@ class Utils(object):
             logger.warning("Please remove 'refresh' parameter.")
 
         self.get_schema()
-        self.connector.print_last_response()
+        self.client.print_last_response()
 
     def get_schema(self, refresh=False):
         """
@@ -175,14 +112,14 @@ class Utils(object):
             }
         }]
 
-        res, _ = self.connector.query(query)
+        res, _ = self.execute(query)
 
         schema = {}
 
         try:
             schema = res[0]["GetSchema"]
         except BaseException as e:
-            logger.error(self.connector.get_last_response_str())
+            logger.error(self.client.get_last_response_str())
             raise e
 
         return schema
@@ -213,10 +150,24 @@ class Utils(object):
         Returns:
             source: The visualization of the schema.
         """
+        if not HAS_GRAPHVIZ:
+            raise Exception("graphviz not installed.")
         r = self.get_schema()
 
+        colors = dict(
+            edge="#3A3B9C",
+            entity_background="#2A2E78",
+            entity_foreground="#E2E0F1",
+            property_background="#337EC0",
+            property_foreground="#E2E0F1",
+            connection_background="#5956F1",
+            connection_foreground="#E2E0F1",
+            connection_property_background="#33E1FF",
+            connection_property_foreground="#2A2E78"
+        )
+
         dot = Digraph(comment='ApertureDB Schema Diagram', node_attr={
-                      'shape': 'none', 'fontcolor': '#E2E0F1'}, graph_attr={'rankdir': 'LR'}, edge_attr={'color': '#3A3B9C'})
+                      'shape': 'none'}, graph_attr={'rankdir': 'LR'}, edge_attr={'color': colors['edge']})
 
         # Add entities as nodes and connections as edges
         entities = r['entities']['classes']
@@ -228,25 +179,25 @@ class Utils(object):
             properties = data["properties"]
             table = f'''<
             <TABLE BORDER="0" CELLBORDER="1" CELLSPACING="0">
-            <TR><TD BGCOLOR="#2A2E78" COLSPAN="3"><B>{entity}</B> ({matched:,})</TD></TR>
+            <TR><TD BGCOLOR="{colors["entity_background"]}" COLSPAN="3"><FONT COLOR="{colors["entity_foreground"]}"><B>{entity}</B> ({matched:,})</FONT></TD></TR>
             '''
             for prop, (matched, indexed, typ) in properties.items():
-                table += f'<TR><TD BGCOLOR="#337EC0"><B>{prop.strip()}</B></TD> <TD BGCOLOR="#337EC0">{matched:,}</TD> <TD BGCOLOR="#337EC0">{"Indexed" if indexed else "Unindexed"}, {typ}</TD></TR>'
+                table += f'<TR><TD BGCOLOR="{colors["property_background"]}"><FONT COLOR="{colors["property_foreground"]}"><B>{prop.strip()}</B></FONT></TD> <TD BGCOLOR="{colors["property_background"]}"><FONT COLOR="{colors["property_foreground"]}">{matched:,}</FONT></TD> <TD BGCOLOR="{colors["property_background"]}"><FONT COLOR="{colors["property_foreground"]}">{"Indexed" if indexed else "Unindexed"}, {typ}</FONT></TD></TR>'
             for connection, data in connections.items():
                 if data['src'] == entity:
                     matched = data["matched"]
                     # dictionary from name to (matched, indexed, type)
                     properties = data["properties"]
-                    table += f'<TR><TD BGCOLOR="#5956F1" COLSPAN="3" PORT="{connection}"><B>{connection}</B> ({matched:,})</TD></TR>'
+                    table += f'<TR><TD BGCOLOR="{colors["connection_background"]}" COLSPAN="3" PORT="{connection}"><FONT COLOR="{colors["connection_foreground"]}"><B>{connection}</B> ({matched:,})</FONT></TD></TR>'
                     if properties:
                         for prop, (matched, indexed, typ) in properties.items():
-                            table += f'<TR><TD BGCOLOR="#33E1FF"><B>{prop.strip()}</B></TD> <TD BGCOLOR="#33E1FF">{matched:,}</TD> <TD BGCOLOR="#33E1FF">{"Indexed" if indexed else "Unindexed"}, {typ}</TD></TR>'
+                            table += f'<TR><TD BGCOLOR="{colors["connection_property_background"]}"><FONT COLOR="{colors["connection_property_foreground"]}"><B>{prop.strip()}</B></FONT></TD> <TD BGCOLOR="{colors["connection_property_background"]}"><FONT COLOR="{colors["connection_property_foreground"]}">{matched:,}</FONT></TD> <TD BGCOLOR="{colors["connection_property_background"]}"><FONT COLOR="{colors["connection_property_foreground"]}">{"Indexed" if indexed else "Unindexed"}, {typ}</FONT></TD></TR>'
             table += '</TABLE>>'
             dot.node(entity, label=table)
 
         for connection, data in connections.items():
             dot.edge(f'{data["src"]}:{connection}',
-                     f'{data["dst"]}:{connection}')
+                     f'{data["dst"]}')
 
         # Render the diagram inline
         s = Source(dot.source, filename="schema_diagram.gv", format="png")
@@ -317,7 +268,7 @@ class Utils(object):
             connections_classes = [c for c in r["connections"]["classes"]]
 
         print(f"================== Summary ==================")
-        print(f"Database: {self.connector.host}")
+        print(f"Database: {self.client.host}")
         print(f"Version: {version}")
         print(f"Status:  {status}")
         print(f"Info:    {info}")
@@ -522,7 +473,8 @@ class Utils(object):
 
         return total_connections
 
-    def add_descriptorset(self, name: str, dim: int, metric="L2", engine="FaissFlat") -> bool:
+    def add_descriptorset(self, name: str, dim: int, metric="L2", engine="FaissFlat",
+                          properties: Optional[Dict] = None) -> bool:
         """
         Add a descriptor set to the database.
 
@@ -531,6 +483,7 @@ class Utils(object):
             dim (int): The dimension of the descriptors.
             metric (str, optional): The metric to use for the descriptors.
             engine (str, optional): The engine to use for the descriptors.
+            properties (dict, optional): The properties of the descriptor set.
 
         Returns:
             success (bool): True if the operation was successful, False otherwise.
@@ -544,6 +497,9 @@ class Utils(object):
             }
         }]
 
+        if properties is not None:
+            query[0]["AddDescriptorSet"]["properties"] = properties
+
         try:
             self.execute(query)
         except:
@@ -639,38 +595,42 @@ class Utils(object):
 
         total = count
 
-        pb = ProgressBar.ProgressBar()
+        pb = tqdm(total=total, desc=f"Removing {type} {class_name}",
+                  unit="elements", unit_scale=True, dynamic_ncols=True)
 
         find = "Find" + cmd
         dele = "Delete" + cmd
-        while count > 0:
+        try:
+            while count > 0:
 
-            q = [{
-                find: {
-                    "_ref": 1,
-                    "with_class": class_name,
-                    "results": {
-                        "limit": batch_size
+                q = [{
+                    find: {
+                        "_ref": 1,
+                        "with_class": class_name,
+                        "results": {
+                            "limit": batch_size
+                        }
                     }
-                }
-            }, {
-                dele: {
-                    "ref": 1
-                }
-            }]
+                }, {
+                    dele: {
+                        "ref": 1
+                    }
+                }]
 
-            try:
-                self.execute(q)
-            except:
-                return False
+                try:
+                    self.execute(q)
+                except:
+                    return False
 
-            self.print(
-                f"Delete transaction of {batch_size} elements took: {self.connector.get_last_query_time()}")
+                self.print(
+                    f"Delete transaction of {batch_size} elements took: {self.client.get_last_query_time()}")
 
-            count -= batch_size
+                count -= batch_size
 
-            if self.verbose:
-                pb.update(abs((count - total) / total))
+                if self.verbose:
+                    pb.update(batch_size)
+        finally:
+            pb.close()
 
         return True
 
@@ -705,7 +665,7 @@ class Utils(object):
             count = r[0]["DeleteEntity"]["count"]
             self.print(f"Deleted {count} entities")
             self.print(
-                f"Delete transaction took: {self.connector.get_last_query_time()}")
+                f"Delete transaction took: {self.client.get_last_query_time()}")
         except:
             logger.error("Could not get count of deleted entities")
             return False
@@ -743,7 +703,7 @@ class Utils(object):
             count = r[0]["DeleteConnection"]["count"]
             self.print(f"Deleted {count} connections")
             self.print(
-                f"Delete transaction took: {self.connector.get_last_query_time()}")
+                f"Delete transaction took: {self.client.get_last_query_time()}")
         except:
             logger.error("Could not get count of deleted connections")
             return False
@@ -936,7 +896,8 @@ class Utils(object):
         """
         Log a message to the user log.
 
-        This is useful because it can later be seen in Grafana, not only as log entries in the AperturDB Logging dashboard, but also as event markers in the Aperture DB Status dahsboard.
+        This is useful because it can later be seen in Grafana, not only as log entries in the ApertureDB
+        Logging dashboard, but also as event markers in the ApertureDB Status dashboard.
 
         Args:
             message (str): The message to log.
@@ -946,3 +907,10 @@ class Utils(object):
                          "ERROR"], f"Invalid log level: {level}"
         q = [{"UserLogMessage": {"text": message, "type": level}}]
         self.execute(q)
+
+
+def create_connector():
+    from aperturedb.CommonLibrary import create_connector, issue_deprecation_warning
+    issue_deprecation_warning("Utils.create_connector",
+                              "CommonLibrary.create_connector")
+    return create_connector()
