@@ -2,7 +2,7 @@ import typer
 
 from aperturedb.cli import configure, ingest, utilities, transact
 
-app = typer.Typer()
+app = typer.Typer(pretty_exceptions_show_locals=False)
 
 app.add_typer(ingest.app, name="ingest", help="Ingest data into ApertureDB.")
 app.add_typer(configure.app, name="config",
