@@ -4,13 +4,12 @@ from aperturedb.BoundingBoxes import BoundingBoxes
 from aperturedb.Constraints import Constraints
 from aperturedb.Entities import Entities
 from aperturedb.Images import Images
-from aperturedb.Query import ObjectType, Query
-from aperturedb.Query import generate_save_query
-from aperturedb.QueryTypes import ImageModel, IdentityModel
+from aperturedb.Query import Query
+from aperturedb.Query import generate_add_query
+from aperturedb.DataModels import ImageDataModel, IdentityDataModel
 import random
 from typing import List
 from enum import Enum
-from devtools import debug
 
 logger = logging.getLogger(__file__)
 
@@ -102,21 +101,21 @@ def make_people(count: int = 1) -> List[object]:
         RIGHT = 1
         LEFT = 2
 
-    class Finger(IdentityModel):
+    class Finger(IdentityDataModel):
         nail_clean: bool = False
 
-    class Hand(ImageModel):
+    class Hand(ImageDataModel):
         side: Side = None
         thumb: Finger = None
         fingers: List[Finger] = []
 
-    class Person(IdentityModel):
+    class Person(IdentityDataModel):
         name: str = ""
         hands: List[Hand] = []
         dominant_hand: Hand = None
 
     def make_hand(side: Side) -> Hand:
-        hand = Hand(side = side, file= "input/images/0079.jpg")
+        hand = Hand(side = side, url= "input/images/0079.jpg")
         hand.fingers = [Finger(nail_clean=True) if random.randint(
             0, 1) == 1 else Finger(nail_clean=False) for i in range(5)]
         hand.thumb = hand.fingers[0]
@@ -146,7 +145,7 @@ class TestQueryBuilder():
         total_commands = []
         total_blobs = []
         for person in people:
-            q, b, current_ref = generate_save_query(person)
+            q, b, current_ref = generate_add_query(person)
             total_commands += q
             total_blobs += b
             # 16 entities have been inserted and referenced
